using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;

//  Auto Bulger Plugin by Saking55
//
//  A modified version of the Belly Bulger Plugin by Captain Varghoss.
//
//  It basically works like the Captain Varghoss plugin, but it 
//  selects the target automatically.
//
//  Changes have also been made to the way distances are calculated.
//  Distances are calculated from the Rigid Body that is colliding 
//  rather than the position of the main object or controller.
//
//  Morphings are only applied if the corresponding triggers are in true state.
//
//  Added a smoothing filter to reduce flickering of morphs.
//
//	Morphs not included.  
//
//  
//  All credit goes to the original creators of these plugins: 
//  VRAdultFun and further enhanced by Captain Varghoss. 
//

namespace Saking55
{
    public class AutoBellyBulger : MVRScript
    {
        protected JSONStorableBool isActive;
        protected JSONStorableString uiFocusTarget;
        protected JSONStorableFloat uiBulgeMult;
        protected JSONStorableFloat uiMinBulgeDistMult;
        protected JSONStorableFloat uiBulge1Mult;
        protected JSONStorableFloat uiBulge2Mult;
        protected JSONStorableFloat uiBulge3Mult;
        protected JSONStorableFloat uiBulge4Mult;
        protected JSONStorableFloat uiBulge5Mult;
        protected JSONStorableFloat uiBulge6Mult;
        protected JSONStorableFloat uiBulge7Mult;
        protected JSONStorableFloat uiBulge8Mult;

        protected JSONStorableFloat uiToyMulti;
        protected JSONStorableFloat uiCurrentDepth;
        protected UIDynamicTextField helpTextfield;
        protected JSONStorableString helpText;
        protected UIDynamicTextField helpTextfield2;
        protected JSONStorableString helpText2;
        protected JSONStorableString stateInfos;

        private static float tempFloat = 1.0f;

        private static FreeControllerV3 pelvisController;
        private static DAZMorph morphDeepBulgeBelly1;
        private static DAZMorph morphDeepBulgeBelly2;
        private static DAZMorph morphDeepBulgeBelly3;
        private static DAZMorph morphDeepBulgeBelly4;
        private static DAZMorph morphDeepBulgeBelly5;
        private static DAZMorph morphDeepBulgeBelly6;
        private static DAZMorph morphDeepBulgeBelly7;
        private static DAZMorph morphDeepBulgeBelly8;

        private static Atom person;
        private static Atom currentAtom;

        CollisionTrigger colTrigger;
        CollisionTriggerEventHandler triggerEvtHdl;
        String targetAtomName;

        private CollisionTrigger triggerVag;
        private CollisionTrigger triggerVagD;
        private CollisionTrigger triggerVagDD;

        private Collider temp;
        private Collider colDistTarget = null;

        float yVelocity = 0.0f;

        public override void Init()
        {
            try
            {

                // Active Toggle
                isActive = new JSONStorableBool("isActive", true);
                RegisterBool(isActive);
                var toggle = CreateToggle(isActive);
                toggle.label = "Active";	

                JSONStorableAction ToggleActive = new JSONStorableAction("Toggle Belly Bulger", () =>
                {
                    isActive.val = !isActive.val;
                });
                RegisterAction(ToggleActive);
                List<string> targetChoices = new List<string>();
                foreach (string atomUID in SuperController.singleton.GetAtomUIDs())
                {
                    currentAtom = SuperController.singleton.GetAtomByUid(atomUID);
                    if (currentAtom != containingAtom && atomUID != null)
                    {
                        targetChoices.Add(atomUID);
                    }
                }

                // uiFocusTarget = new JSONStorableStringChooser("Sex Giver", targetChoices, "None", "Choose Person");
                // RegisterStringChooser(uiFocusTarget);
				// UIDynamicPopup udp = CreateScrollablePopup(uiFocusTarget, true);
				
				helpText = new JSONStorableString("Helptext", "\n<b>Auto Belly Bulger</b>\n\nAutomatically selects the collinding object and apply morphs to Person.\n");
                UIDynamic helpTextfield = CreateTextField(helpText, true);
                helpTextfield.height = 210f;

				uiFocusTarget = new JSONStorableString( "TargetObject","\n<b>Target:</b> None");
				UIDynamic targetText = CreateTextField(uiFocusTarget, true);
				targetText.height = 100f;
                //helpTextfield.height = 700f;

                uiBulgeMult = new JSONStorableFloat("Belly Bulge Multiplier", 1f, 0.0f, 3.0f, true, true);
                RegisterFloat(uiBulgeMult);
                CreateSlider(uiBulgeMult, false);

                uiMinBulgeDistMult = new JSONStorableFloat("Min Bulge Dist Multiplier", 5.0f, 0.0f, 5.0f, true, true);
                RegisterFloat(uiMinBulgeDistMult);
                CreateSlider(uiMinBulgeDistMult, false);

                uiToyMulti = new JSONStorableFloat("Toy Distance Multiplier", 5.0f, 0.0f, 5.0f, true, true);
                RegisterFloat(uiToyMulti);
                //				CreateSlider(uiToyMulti, false);

                uiCurrentDepth = new JSONStorableFloat("Current Depth", 0.0f, 0f, 1.0f, true, true);
                RegisterFloat(uiCurrentDepth);
                CreateSlider(uiCurrentDepth, false);

                var btn = CreateButton("Reset Morphs", false);
                btn.button.onClick.AddListener(() => { ResetMorphs(); });


				
                //				stateInfos = new JSONStorableString("Stateinfos", "");
                //				UIDynamic stateinfosTextfield = CreateTextField(stateInfos, false);
                //				stateinfosTextfield.height = 180.0f;
				
                // Per morph multiplier reducers
                helpText2 = new JSONStorableString("Helptext2", "\n<b>Bulge Morph Adjustments</b>\n\nUse these morphs to reduce Bulge Multiplier on individual morphs.");
                UIDynamic helpTextfield2 = CreateTextField(helpText2, false);
                helpTextfield2.height = 200f;



                uiBulge1Mult = new JSONStorableFloat("Bulge 1 Amount Mult", 0.32f, 0.0f, 1.0f, true, true);
                RegisterFloat(uiBulge1Mult);
                CreateSlider(uiBulge1Mult, true);

                uiBulge2Mult = new JSONStorableFloat("Bulge 2 Amount Mult", 0.67f, 0.0f, 1.0f, true, true);
                RegisterFloat(uiBulge2Mult);
                CreateSlider(uiBulge2Mult, true);

                uiBulge3Mult = new JSONStorableFloat("Bulge 3 Amount Mult", 0.9f, 0.0f, 1.0f, true, true);
                RegisterFloat(uiBulge3Mult);
                CreateSlider(uiBulge3Mult, true);

                uiBulge4Mult = new JSONStorableFloat("Bulge 4 Amount Mult", 1.0f, 0.0f, 1.0f, true, true);
                RegisterFloat(uiBulge4Mult);
                CreateSlider(uiBulge4Mult, true);

                uiBulge5Mult = new JSONStorableFloat("Bulge 5 Amount Mult", 0.86f, 0.0f, 1.0f, true, true);
                RegisterFloat(uiBulge5Mult);
                CreateSlider(uiBulge5Mult, true);

                uiBulge6Mult = new JSONStorableFloat("Bulge 6 Amount Mult", 0.7f, 0.0f, 1.0f, true, true);
                RegisterFloat(uiBulge6Mult);
                CreateSlider(uiBulge6Mult, true);

                uiBulge7Mult = new JSONStorableFloat("Bulge 7 Amount Mult", 0.39f, 0.0f, 1.0f, true, true);
                RegisterFloat(uiBulge7Mult);
                CreateSlider(uiBulge7Mult, true);

                uiBulge8Mult = new JSONStorableFloat("Bulge 8 Amount Mult", 0.18f, 0.0f, 1.0f, true, true);
                RegisterFloat(uiBulge8Mult);
                CreateSlider(uiBulge8Mult, true);
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        void Start()
        {
            try
            {
                person = containingAtom;
                if (person != null)
                {
                    triggerVag = (containingAtom.GetStorableByID("VaginaTrigger") as CollisionTrigger);
                    triggerVagD = (containingAtom.GetStorableByID("DeepVaginaTrigger") as CollisionTrigger);
                    triggerVagDD = (containingAtom.GetStorableByID("DeeperVaginaTrigger") as CollisionTrigger);

                    triggerEvtHdl = triggerVag.GetComponent(typeof(CollisionTriggerEventHandler)) as CollisionTriggerEventHandler;


                    pelvisController = person.GetStorableByID("pelvisControl") as FreeControllerV3;
                    ResetMorphs();
                }
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        private void ResetMorphs()
        {
            JSONStorable js = person.GetStorableByID("geometry");
            DAZCharacterSelector dcs = js as DAZCharacterSelector;
            GenerateDAZMorphsControlUI morphUI = dcs.morphsControlUI;

            if (morphUI != null)
            {
                morphDeepBulgeBelly1 = morphUI.GetMorphByDisplayName("Belly Bulger 1");
                morphDeepBulgeBelly1.Reset();
                morphDeepBulgeBelly2 = morphUI.GetMorphByDisplayName("Belly Bulger 2");
                morphDeepBulgeBelly2.Reset();
                morphDeepBulgeBelly3 = morphUI.GetMorphByDisplayName("Belly Bulger 3");
                morphDeepBulgeBelly3.Reset();
                morphDeepBulgeBelly4 = morphUI.GetMorphByDisplayName("Belly Bulger 4");
                morphDeepBulgeBelly4.Reset();
                morphDeepBulgeBelly5 = morphUI.GetMorphByDisplayName("Belly Bulger 5");
                morphDeepBulgeBelly5.Reset();
                morphDeepBulgeBelly6 = morphUI.GetMorphByDisplayName("Belly Bulger 6");
                morphDeepBulgeBelly6.Reset();
                morphDeepBulgeBelly7 = morphUI.GetMorphByDisplayName("Belly Bulger 7");
                morphDeepBulgeBelly7.Reset();
                morphDeepBulgeBelly8 = morphUI.GetMorphByDisplayName("Belly Bulger 8");
                morphDeepBulgeBelly8.Reset();
            }
        }

        void FixedUpdate()
        {
            if (isActive.val == true)
            {

                try
                {

                    targetAtomName = "None";
                    foreach (KeyValuePair<Collider, bool> pair in triggerEvtHdl.collidingWithDictionary)
                    {
                        targetAtomName = (pair.Key.attachedRigidbody.GetComponentInParent(typeof(Atom)) as Atom).name;
                        if (targetAtomName.Equals(containingAtom.name))
                        {
                            targetAtomName = "None";
                        }else{
                            temp = pair.Key;
                            break;
                        }
                    }
                    if (colDistTarget == null)
                        colDistTarget = temp;
                    if (targetAtomName == "None")
                        colDistTarget = null;


                    //helpText.val = "Target Name: <b>" + targetAtomName + "</b>\n";//
                    //helpText.val += "Pen Value: <b>" + tempFloat + "</b>\n";
                    uiFocusTarget.val = "\n<b>Target:</b> "+ targetAtomName;

                    if (morphDeepBulgeBelly1 != null && morphDeepBulgeBelly2 != null && morphDeepBulgeBelly3 != null && morphDeepBulgeBelly4 != null && morphDeepBulgeBelly5 != null && morphDeepBulgeBelly6 != null && morphDeepBulgeBelly7 != null && morphDeepBulgeBelly8 != null)
                    {
                        if (targetAtomName.Equals("None"))
                            tempFloat = Mathf.SmoothDamp(tempFloat, 1f, ref yVelocity, 0.5f);
                        else if (triggerVag.trigger.active || triggerVagD.trigger.active || triggerVagDD.trigger.active)
                        {
                            //helpText.val += "Distance: <b>" + Vector3.Distance(pelvisController.followWhenOff.position, colDistTarget.attachedRigidbody.position) + "</b>";
                            tempFloat = Mathf.SmoothDamp(tempFloat, remapp(Vector3.Distance(pelvisController.followWhenOff.position, colDistTarget.attachedRigidbody.position), 0.05f, 0.2f, 0f, 0.7f), ref yVelocity, 0.02f);
                        }

                        // Morph 1
                        if (tempFloat < 0.15f * uiMinBulgeDistMult.val)
                        {
                            morphDeepBulgeBelly1.morphValue = Mathf.Clamp((1.0f - (tempFloat * (6.666f / uiMinBulgeDistMult.val))) * (uiBulgeMult.val * uiBulge1Mult.val), 0.0f, 1.0f);
                        }
                        else
                        {
                            morphDeepBulgeBelly1.Reset();
                        }
                        // Morph 2
                        if (tempFloat < 0.13f * uiMinBulgeDistMult.val)
                        {
                            morphDeepBulgeBelly2.morphValue = Mathf.Clamp((1.0f - (tempFloat * (12f / uiMinBulgeDistMult.val))) * (uiBulgeMult.val * uiBulge2Mult.val), 0.0f, 1.0f);
                        }
                        else
                        {
                            morphDeepBulgeBelly2.Reset();
                        }
                        // Morph 3
                        if (tempFloat < 0.11f * uiMinBulgeDistMult.val)
                        {
                            morphDeepBulgeBelly3.morphValue = Mathf.Clamp((1.0f - (tempFloat * (12f / uiMinBulgeDistMult.val))) * (uiBulgeMult.val * uiBulge3Mult.val), 0.0f, 1.0f);
                        }
                        else
                        {
                            morphDeepBulgeBelly3.Reset();
                        }
                        // Morph 4
                        if (tempFloat < 0.09f * uiMinBulgeDistMult.val)
                        {
                            morphDeepBulgeBelly4.morphValue = Mathf.Clamp((1.0f - (tempFloat * (18f / uiMinBulgeDistMult.val))) * (uiBulgeMult.val * uiBulge4Mult.val), 0.0f, 1.0f);
                        }
                        else
                        {
                            morphDeepBulgeBelly4.Reset();
                        }
                        // Morph 5
                        if (tempFloat < 0.07f * uiMinBulgeDistMult.val)
                        {
                            morphDeepBulgeBelly5.morphValue = Mathf.Clamp((1.0f - (tempFloat * (24f / uiMinBulgeDistMult.val))) * (uiBulgeMult.val * uiBulge5Mult.val), 0.0f, 1.0f);
                        }
                        else
                        {
                            morphDeepBulgeBelly5.Reset();
                        }
                        // Morph 6
                        if (tempFloat < 0.05f * uiMinBulgeDistMult.val)
                        {
                            morphDeepBulgeBelly6.morphValue = Mathf.Clamp((1.0f - (tempFloat * (30f / uiMinBulgeDistMult.val))) * (uiBulgeMult.val * uiBulge6Mult.val), 0.0f, 1.0f);
                        }
                        else
                        {
                            morphDeepBulgeBelly6.Reset();
                        }
                        // Morph 7
                        if (tempFloat < 0.03f * uiMinBulgeDistMult.val)
                        {
                            morphDeepBulgeBelly7.morphValue = Mathf.Clamp((1.0f - (tempFloat * (36f / uiMinBulgeDistMult.val))) * (uiBulgeMult.val * uiBulge7Mult.val), 0.0f, 1.0f);
                        }
                        else
                        {
                            morphDeepBulgeBelly7.Reset();
                        }
                        // Morph 8
                        if (tempFloat < 0.025f * uiMinBulgeDistMult.val)
                        {
                            morphDeepBulgeBelly8.morphValue = Mathf.Clamp((1.0f - (tempFloat * (42f / uiMinBulgeDistMult.val))) * (uiBulgeMult.val * uiBulge8Mult.val), 0.0f, 1.0f);
                        }
                        else
                        {
                            morphDeepBulgeBelly8.Reset();
                        }
                    }
                    uiCurrentDepth.val = tempFloat;
                }
                catch (Exception e)
                {
                    SuperController.LogError("Exception caught: " + e);
                }
            }
            else { ResetMorphs(); }
        }


        private float remapp(float val, float from1, float to1, float from2, float to2)
        {
            return (val - from1) / (to1 - from1) * (to2 - from2) + from2;
        }
        void OnDestroy()
        {
        }

    }
}