﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;

//  Auto Bulger Plugin by Saking55
//
//  A modified version of the Throat Bulger Plugin by Captain Varghoss.
//
//  It basically works like the Captain Varghoss plugin, but it 
//  selects the target automatically.
//
//  Changes have also been made to the way distances are calculated.
//  Distances are calculated from the Rigid Body that is colliding 
//  rather than the position of the main object or controller.
//
//  Morphings are only applied if the corresponding triggers are in true state.
//
//  Added a smoothing filter to reduce flickering of morphs.
//
//	Morphs not included.  
//
//
//  All credit goes to the original creators of these plugins: 
//  VRAdultFun and further enhanced by Captain Varghoss. 
//
// 

namespace Saking55
{
    public class AutoThroatBulger : MVRScript
    {
        protected JSONStorableBool isActive;
        protected JSONStorableString uiFocusTarget;
        protected JSONStorableFloat uiBulgeMult;
        protected JSONStorableFloat uiMinBulgeDistMult;
        protected JSONStorableFloat uiBulge1Mult;
        protected JSONStorableFloat uiBulge2Mult;
        protected JSONStorableFloat uiBulge3Mult;

        protected JSONStorableFloat uiToyMulti;
        protected JSONStorableFloat uiCurrentDepth;
        protected UIDynamicTextField helpTextfield;
        protected JSONStorableString helpText;
        protected UIDynamicTextField helpTextfield2;
        protected JSONStorableString helpText2;
        protected JSONStorableString stateInfos;

        private static float toyFloat = 0.0f;
        private static float tempFloat = 1.0f;

        private static FreeControllerV3 headController;
        private static FreeControllerV3 pelvisController;
        private static DAZMorph morphDeepBulgeThroat1;
        private static DAZMorph morphDeepBulgeThroat2;
        private static DAZMorph morphDeepBulgeThroat3;

        private static Atom person;
        private static Atom person2;
        private static Atom currentAtom;
        private static string currentAtomName = "None";
        private static FreeControllerV3 playerPelvisController;
        private static FreeControllerV3 playerHeadController;
        private static FreeControllerV3 playerTipController;
        private static FreeControllerV3 playerTipBaseController;
        private static bool isObject = false;

        private CollisionTrigger colTrigger;
        private CollisionTriggerEventHandler triggerEvtHdl;
        private String targetAtomName;

        private CollisionTrigger triggerMouth;
        private CollisionTrigger triggerThroat;
        private Collider colDistTarget;

        float yVelocity = 0.0f;

        Collider temp;

        public override void Init()
        {
            try
            {
                // Active Toggle
                isActive = new JSONStorableBool("isActive", true);
                RegisterBool(isActive);
                var toggle = CreateToggle(isActive);
                toggle.label = "Active";

                JSONStorableAction ToggleActive = new JSONStorableAction("Toggle Throat Bulger", () =>
                {
                    isActive.val = !isActive.val;
                });
                RegisterAction(ToggleActive);
                List<string> targetChoices = new List<string>();
                foreach (string atomUID in SuperController.singleton.GetAtomUIDs())
                {
                    currentAtom = SuperController.singleton.GetAtomByUid(atomUID);
                    if (currentAtom != containingAtom && atomUID != null)
                    {
                        targetChoices.Add(atomUID);
                    }
                }
				helpText = new JSONStorableString("Helptext", "\n<b>Auto Throat Bulger</b>\n\nAutomatically selects the collinding object and apply morphs to Person.\n");
                UIDynamic helpTextfield = CreateTextField(helpText, true);
                helpTextfield.height = 210f;

				uiFocusTarget = new JSONStorableString( "TargetObject","\n<b>Target:</b> None");
				UIDynamic targetText = CreateTextField(uiFocusTarget, true);
				targetText.height = 100f;
                //helpTextfield.height = 700f;

                uiBulgeMult = new JSONStorableFloat("Throat Bulge Multiplier", 1f, 0.0f, 1.0f, true, true);
                RegisterFloat(uiBulgeMult);
                CreateSlider(uiBulgeMult, false);

                uiMinBulgeDistMult = new JSONStorableFloat("Min Bulge Dist Multiplier", 2f, 0.0f, 5.0f, true, true);
                RegisterFloat(uiMinBulgeDistMult);
                CreateSlider(uiMinBulgeDistMult, false);

                uiToyMulti = new JSONStorableFloat("Toy Distance Multiplier", 5.0f, 0.0f, 5.0f, true, true);
                RegisterFloat(uiToyMulti);

                uiCurrentDepth = new JSONStorableFloat("Current Depth", 0.0f, 0f, 1.0f, true, true);
                RegisterFloat(uiCurrentDepth);
                CreateSlider(uiCurrentDepth);

                var btn = CreateButton("Reset Morphs", false);
                btn.button.onClick.AddListener(() => { ResetMorphs(); });

                // Per morph multiplier reducers
                helpText2 = new JSONStorableString("Helptext2", "\n<b>Bulge Morph Adjustments</b>\n\nUse these morphs to reduce Bulge Multiplier on individual morphs.");
                UIDynamic helpTextfield2 = CreateTextField(helpText2, false);
                helpTextfield2.height = 200f;

                uiBulge1Mult = new JSONStorableFloat("Bulge 1 Amount Mult", 1.0f, 0.0f, 1.0f, true, true);
                RegisterFloat(uiBulge1Mult);
                CreateSlider(uiBulge1Mult, false);

                uiBulge2Mult = new JSONStorableFloat("Bulge 2 Amount Mult", 1.0f, 0.0f, 1.0f, true, true);
                RegisterFloat(uiBulge2Mult);
                CreateSlider(uiBulge2Mult, false);

                uiBulge3Mult = new JSONStorableFloat("Bulge 3 Amount Mult", 1.0f, 0.0f, 1.0f, true, true);
                RegisterFloat(uiBulge3Mult);
                CreateSlider(uiBulge3Mult, false);
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        void Start()
        {
            try
            {
                person = containingAtom;
                if (person != null)
                {
                    triggerMouth = (containingAtom.GetStorableByID("MouthTrigger") as CollisionTrigger);
                    triggerThroat = (containingAtom.GetStorableByID("ThroatTrigger") as CollisionTrigger);
                    triggerEvtHdl = triggerMouth.GetComponent(typeof(CollisionTriggerEventHandler)) as CollisionTriggerEventHandler;

                    headController = person.GetStorableByID("neckControl") as FreeControllerV3;
                    pelvisController = person.GetStorableByID("hipControl") as FreeControllerV3;
                    ResetMorphs();
                }
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        private void ResetMorphs()
        {
            JSONStorable js = person.GetStorableByID("geometry");
            DAZCharacterSelector dcs = js as DAZCharacterSelector;
            GenerateDAZMorphsControlUI morphUI = dcs.morphsControlUI;

            if (morphUI != null)
            {
                morphDeepBulgeThroat1 = morphUI.GetMorphByDisplayName("Throat Bulge 1");
                morphDeepBulgeThroat1.Reset();
                morphDeepBulgeThroat2 = morphUI.GetMorphByDisplayName("Throat Bulge 2");
                morphDeepBulgeThroat2.Reset();
                morphDeepBulgeThroat3 = morphUI.GetMorphByDisplayName("Throat Bulge 3");
                morphDeepBulgeThroat3.Reset();
            }
        }

        void FixedUpdate()
        {
            if (isActive.val == true)
            {
                try
                {

                    targetAtomName = "None";
                    foreach (KeyValuePair<Collider, bool> pair in triggerEvtHdl.collidingWithDictionary)
                    {
                        targetAtomName = (pair.Key.attachedRigidbody.GetComponentInParent(typeof(Atom)) as Atom).name;

                        if (targetAtomName.Equals(containingAtom.name))
                        {
                            targetAtomName = "None";
                        }else{
                            temp = pair.Key;
                            break;
                        }
                    }
                    if (colDistTarget == null)
                        colDistTarget = temp;
                    if (targetAtomName == "None")
                        colDistTarget = null;

                    //helpText.val = "Target Name: <b>" + targetAtomName + "</b>\n";
                    //helpText.val += "Pen Value: <b>" + tempFloat + "</b>\n";
                    uiFocusTarget.val = "\n<b>Target:</b> "+ targetAtomName;

                    currentAtomName = uiFocusTarget.val;
                    if (morphDeepBulgeThroat1 != null && morphDeepBulgeThroat2 != null && morphDeepBulgeThroat3 != null)
                    {
                        if (targetAtomName.Equals("None"))
                            tempFloat = Mathf.SmoothDamp(tempFloat, 1f, ref yVelocity, 0.3f);
                        else if (triggerMouth.trigger.active || triggerThroat.trigger.active)
                        {
                            //helpText.val += "Distance: <b>" + Vector3.Distance(headController.followWhenOff.position, colDistTarget.attachedRigidbody.position) + "</b>";
                            tempFloat = Mathf.SmoothDamp(tempFloat, remapp(Vector3.Distance(headController.followWhenOff.position, colDistTarget.attachedRigidbody.position), 0.085f, 0.16f, 0f, 0.3f), ref yVelocity, 0.1f);
                        }

                        // Morph 1
                        if (tempFloat < 0.15f * uiMinBulgeDistMult.val)
                        {
                            morphDeepBulgeThroat1.morphValue = Mathf.Clamp((1.0f - (tempFloat * (6.666f / uiMinBulgeDistMult.val))) * (uiBulgeMult.val * uiBulge1Mult.val), 0.0f, 1.0f);
                        }
                        else
                        {
                            morphDeepBulgeThroat1.Reset();
                        }
                        // Morph 2
                        if (tempFloat < 0.12f * uiMinBulgeDistMult.val)
                        {
                            morphDeepBulgeThroat2.morphValue = Mathf.Clamp((1.0f - (tempFloat * (16f / uiMinBulgeDistMult.val))) * (uiBulgeMult.val * uiBulge2Mult.val), 0.0f, 1.0f);
                        }
                        else
                        {
                            morphDeepBulgeThroat2.Reset();
                        }
                        // Morph 3
                        if (tempFloat < 0.7f * uiMinBulgeDistMult.val)
                        {
                            morphDeepBulgeThroat3.morphValue = Mathf.Clamp((1.0f - (tempFloat * (22f / uiMinBulgeDistMult.val))) * (uiBulgeMult.val * uiBulge3Mult.val), 0.0f, 1.0f);
                        }
                        else
                        {
                            morphDeepBulgeThroat3.Reset();
                        }
                    }

                    uiCurrentDepth.val = tempFloat;
                }
                catch (Exception e)
                {
                    SuperController.LogError("Exception caught: " + e);
                }
            }
            else { ResetMorphs(); }
        }

        private float remapp(float val, float from1, float to1, float from2, float to2)
        {
            return (val - from1) / (to1 - from1) * (to2 - from2) + from2;
        }

        void OnDestroy()
        {
        }

    }
}